import networkx as nx
import itertools

def getWidth(order):
    '''
    Given a graph G, and it's edge ordering, returns
    the linear width of that ordering.
    
    INPUT
    order: An edge ordering of G

    OUTPUT
    width: The width of order
    '''
    #We make a left and the right set of nodes incident to 1-j (left) 
    #and j + 1 - len(order) (right)

    left = set()
    right = set.union(*order)
    width = len(left.intersection(right))

    #Move the sets along by one and construct the new node lists
    for i in xrange(len(order) - 1):
        left = left.union(order[i])
        right = set.union(*(order[i + 1:]))
        #print 'For i = ', i, 'left = ', left, 'and right= ', right

        newWidth = len(left.intersection(right))

        if newWidth > width:
            width = newWidth

    return width

def linK(G, k):
    '''
    Given a graph and a positive integer k, checks whether
    the linear width of G <= k. We proceed to build an edge
    ordering by dynamic programming, and eliminate the orderings
    above k.

    INPUT
    G: A networkx graph
    k: A positive integer

    OUTPUT
    less: A boolean which is True if lin-width <= k and False otherwise.
    '''
    #We will store the orderings here as [ordering, width]
    FSi = [ [[],0] ]
    
    edges = G.edges()

    #Add edges one at a time to orderings still below k
    for edge in edges:
        newFSi = []

        for orderPair in FSi:
            order = orderPair[0][:]

            for i in xrange(0, len(order) + 1):
                #Make a new ordering by inserting the edge
                
                order = orderPair[0][:]
                order.insert(i, set(edge))

                #if the width <= k put in the new characteristic set
                newWidth = getWidth(order)
                
                if newWidth <= k:
                    newFSi.append([order, newWidth])
        
        #if newFSi is empty after adding our linear width is higher
        if len(newFSi) == 0:
            less = False
            return less

        FSi = newFSi[:]
        print 'Made a new FSi with length: ', len(FSi)

    less = True
    return less

def linearWidth(G):
    '''
    Given a graph, calculates the linear width of the graph
    by using the dynamic programming solution multiple times.

    INPUT
    G: A networkx graph

    OUTPUT
    width: The linear width of G
    '''
    #Check k until we find one where lin-width is less than k
    width = 1
    #print 'Starting a new round to check width: ', width, '\n'

    while not linK(G, width):
        width = width + 1
        #print 'Starting a new round to check width: ', width, '\n'

    return width



#Calculates the mixed search number of a graph G
import networkx as nx
from bruteForce import linearWidth

def pendantNodes(G):
    '''
    Given a graph G, finds it's pendant nodes and
    the edge connecting them to another node.

    INPUT
    G: A networkx graph

    OUTPUT
    pendant: A list of the format [[vertex, edge] ... ]
    '''
    nodes = G.nodes()
    
    pendant = []
    
    #Find vertices with degree 1
    for node in nodes:

        if G.degree(node) == 1:
            pendant.append([node, G.edges(node)[0]])
   
    return pendant

def subdivide(G, pendant):
    '''
    Given a graph with a list of pendant nodes (and their edges),
    subdivides each of the pendant edges.

    INPUT
    G: A networkx graph
    pendant: A list of the form [[vertex, edge] ...] where
    vertex is pendant and edge is the only edge incident to
    vertex

    OUTPUT
    Gh: The subdivision of G
    '''
    Gh = G.copy()
    
    #Perform the subdivision for each pendant vertex
    for pendantEdge in pendant:
        
        vertex = pendantEdge[0]
        edge = pendantEdge[1]

        #Remove the old edge, add a new vertex, and two edges
        Gh.remove_edge(*edge)
        Gh.add_node(str(edge))
        Gh.add_edge(edge[0], str(edge))
        Gh.add_edge(edge[1], str(edge))

    return Gh

def ms(G):
    '''
    Given a graph G, calculates it's mixed search number
    using the linear width graph parameter. ms(G) = lw(G^h),
    where G^h is the graph of G with all of it's pendant edges subdivided.

    INPUT
    G: A networkx graph

    OUTPUT
    msG: The mixed search number of G
    '''

    #We first subdivide the pendant edges of G
    pendant = pendantNodes(G)
    Gh = subdivide(G, pendant)

    msG = linearWidth(Gh)
    return msG




from distutils.core import setup

setup(
    name = 'dynamic_search_number',
    packages = ['dynamic_search_number'],
    version = '1.0.5',
    description = 'A NetworkX package that computes mixed search number and edge search number.',
    long_description = 'A Networkx package that can calculate mixed edge search number and edge search number. The implementation uses dynamic programming to solve the related problem of LINEAR WIDTH to compute the search parameters.',
    license = 'MIT',
    author = 'Anton Afanassiev',
    author_email = 'antonafana@yahoo.ca',
    url = 'https://github.com/Jabbath/Search-Number-Dynamic',
    download_url = 'https://github.com/Jabbath/Search-Number-Dynamic/archive/1.0.5.tar.gz',
    keywords = ['graph', 'theory', 'search', 'number', 'mixed', 'edge', 'linear', 'width', 'pursuit', 'evasion'],
    classifiers = ['Development Status :: 5 - Production/Stable'],
    install_requires = ['networkx']
)



#Returns the edge search number of a graph
import networkx as nx
from mixedSearch import ms
import time

def subdivideEdge(G):
    '''
    Given a graph G, returns a graph in which the
    edges of G are subdivided.

    INPUT
    G: A networkx graph

    OUTPUT
    Ge: The edge subdivision of G
    '''
    Ge = G.copy()

    edges = G.edges()
    
    #Go through every edge, remove it, make a new node, and make two edge
    for edge in edges:
        Ge.remove_edge(*edge)
        Ge.add_node(str(edge))
        Ge.add_edge(edge[0], str(edge))
        Ge.add_edge(edge[1], str(edge))

    return Ge
        

def es(G):
    '''
    Given a graph, returns it's edge search number. Works
    on the principle that es(G) = ms(G^e), where G^e is G
    with all of it's edges subdivided.

    INPUT
    G: A networkx graph

    OUTPUT
    esG: The edge search number of G
    '''
    
    #Subdivide the edges of G
    Ge = subdivideEdge(G)
    esG = ms(Ge)

    return esG

